// Solve the Momentum equation

// Updating poorus density and flux
volScalarField voidFractionRho = voidFraction * rho;
surfaceScalarField voidFractionPhi = fvc::interpolate(voidFraction) * phi;

tmp<fvVectorMatrix> UEqn
(
    fvm::ddt(voidFractionRho, U)
  + fvm::div(voidFractionPhi, U)
  // Corrected in turbulence class
  + turbulence->divDevRhoReff(U)
  // Compressibity correction
  - fvm::Sp(fvc::ddt(voidFractionRho) + fvc::div(voidFractionPhi), U) 
 ==
    fvOptions(rho, U)
);

UEqn().relax();

fvOptions.constrain(UEqn());

if (pimple.momentumPredictor())
{
    // Applying implicit porous resistance source term
    if(porousResistance)
    {
        solve
        (
            UEqn() 
         == 
          // Porous resistance source term
          - fvm::Sp(voidFraction*resistanceCoef, U) 
          - voidFraction * fvc::grad(p)
        );
    }
    else
    {
        solve(UEqn() == -voidFraction * fvc::grad(p));
    }

    fvOptions.correct(U);
    K = 0.5*magSqr(U);
}