/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 2011-2013 OpenFOAM Foundation
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

\*---------------------------------------------------------------------------*/
// Reading porous source formulation type
word porousSourceFormulation_
(
    IOdictionary
    (
        IOobject
        (
            "RASProperties",
            runTime_.constant(),
            mesh_,
            IOobject::MUST_READ_IF_MODIFIED,
            IOobject::NO_WRITE,
            false
        )
    ).lookupOrDefault<word>("porousSourceFormulation", "noFormulation")
);

// Checking for availablility of the model or setting default
if(porousSourceFormulation_ != "deLemons" &&
   porousSourceFormulation_ != "Nakayama" &&
   porousSourceFormulation_ != "Takeda")
{ 
    porousSourceFormulation_ = "deLemons";
}

// Epsilon equation porous source term
volScalarField epsilonSource_
(
    IOobject
    (
        "epsilonSource",
        runTime_.timeName(),
        mesh_
    ),
    mesh_,
    dimensionedScalar("epsilonSource",dimensionSet(1,-1,-4,0,0,0,0), 0.0)
);

// k equation porous source term
 volScalarField kSource_
(
    IOobject
    (
        "kSource",
        runTime_.timeName(),
        mesh_
    ),
    mesh_,
    dimensionedScalar("kSource",dimensionSet(1,-1,-3,0,0,0,0), 0.0)
);

// Pedras and de Lemons formulation
if(porousSourceFormulation_=="deLemons")
{
    // Model constant
    scalar Ck_(0.28);

    // Reading permeability
    const volScalarField permeability_(mesh_.lookupObject<volScalarField>("permeability"));

    // Calculating epsilon source
    epsilonSource_ = Ck_*rho_*epsilon_*voidFraction_*mag(U_)/Foam::sqrt(permeability_);

    // Calculating k source
    kSource_ = Ck_*rho_*k_*voidFraction_*mag(U_)/Foam::sqrt(permeability_);
}

// Nakayama and Kuwahra formulation
if(porousSourceFormulation_=="Nakayama")
{
    // Small scalar for skipping zero division
    scalar tolerance_(1e-5);

    // Particles mean diameter
    dimensionedScalar dp_("dp",dimensionSet(0,1,0,0,0,0,0), lookup("dp"));

    volScalarField epsilonInfinity_
    (
        39.0*Foam::pow(voidFraction_, 2.0)*Foam::pow((1.0-voidFraction_), 2.5)
      * Foam::pow(mag(U_), 3.0)/dp_
    );

    volScalarField kInfinity_
    (
        3.7*Foam::pow(voidFraction_, 1.5)*(1-voidFraction_+tolerance_)
      * Foam::pow(mag(U_), 2.0)
    );

    // Calculating epsilon source
    epsilonSource_ = rho_*Foam::pow(epsilonInfinity_, 2.0)/kInfinity_;

    // Calculating k source
    kSource_ = rho_*epsilonInfinity_;
}

// Takeda formulation
if(porousSourceFormulation_=="Takeda")
{
    // Model constant
    scalar Ck_(0.0413);

    // Reading resistance coefficents
    const volScalarField resistanceCoef_(mesh_.lookupObject<volScalarField>("resistanceCoef"));

    // Calculating epsilon source
    epsilonSource_ = Ck_*resistanceCoef_*epsilon_*voidFraction_*Foam::pow(mag(U_), 2.0)/k_;

    // Calculating k source
    kSource_ = Ck_*resistanceCoef_*Foam::pow(mag(U_), 2.0);
}

// ************************************************************************* //
